import java.io.FileInputStream;
import java.io.FileOutputStream;


public class HTML_Stegano
{
	public static final String hex = "0123456789abcdef";
	
	HTML_StateMachine machine;
	
	public HTML_Stegano()
	{
		machine = new HTML_StateMachine();
	}
	
	public void codeMessage(String in, String out, byte[] message) throws Exception
	{
		// Hossz felfzse a legelejre
		byte[] message2 = new byte[message.length+1];
		message2[0] = (byte) message.length;
		for(int i = 0; i < message.length; i++)
			message2[i+1] = message[i];
		message = message2;

		// Van elg hely?
		int sp = calcSpace(in);
		if(Math.floor(sp/8) < message.length)
		{
			System.out.println("Kicsi a fjl hasznosthat terlete, ekkora zenetet nem lehet belekdolni.");
			System.out.println("Hely: " + sp + " bit (" + Math.floor(sp/8) + " bjt)");
			System.out.println("zenet: " + message.length*8 + " bit (" + message.length + " bjt)");
			return;
		}
		
		// Vltozk inicializlsa
		FileInputStream fi = new FileInputStream(in);
		FileOutputStream fo = new FileOutputStream(out);
		byte b;
		char c, co;

		// Bitfolyam ellltsa
		StringBuffer s = new StringBuffer();
		for(int i = 0; i < message.length; i++)
			s.append(byteToBinString(message[i]));
		String binary = new String(s);

		// A lnyeg
		machine.reset();
		while((b = (byte)fi.read()) != -1)
		{
			machine.input(b);
			if(machine.getOutput() != 0 && (binary.length() > 0))
			{
				c = binary.charAt(0);
				binary = binary.substring(1);

				co = (char) b;
				if(c == '1')
					co = Character.toUpperCase(co);
				else
					co = Character.toLowerCase(co);
					
				fo.write((byte) co);
			}
			else
				fo.write(b);
		}
		
		// ...
		fi.close();
		fo.close();
	}
	
	public int calcSpace(String file) throws Exception
	{
		FileInputStream fs = new FileInputStream(file);
		byte b;
		int size = 0;

		machine.reset();
		while((b = (byte)fs.read()) != -1)
		{
			machine.input(b);
			if(machine.getOutput() != 0)
				size++;
		}
		fs.close();
		
		return size;
	}
	
	public byte[] decodeMessage(String file) throws Exception
	{
		FileInputStream fs = new FileInputStream(file);
		byte b;
		byte[] ba;
		StringBuffer s1 = new StringBuffer();
		char c = 0;

		// Bitfolyam kitermelse
		machine.reset();
		while((b = (byte)fs.read()) != -1)
		{
			machine.input(b);
			if(machine.getOutput() != 0)
			{
				c = (char) b;
				if(Character.isUpperCase(c))
					s1.append("1");
				else
					s1.append("0");
			}
		}
		fs.close();
		
		// Bitfolyam feldolgozsa
		String sub = "", str = new String(s1);
		sub = str.substring(0, 8);
		str = str.substring(8);
		byte length = binStringToByte(sub), ctr = 0;
		ba = new byte[length]; 
		
		while(str.length() > 0 && ctr < length)
		{
			sub = str.substring(0, 8);
			str = str.substring(8);
			ba[ctr] = binStringToByte(sub);
			ctr++;
		}

		return ba;
	}

	// a1fef835078426699f1f23ef4ce19aa5bc72e773067597223c0e975afa4ccdcb8121bbfb67b7156f71b35a0d9f770fe21b818595a64f35bcbaf8228eea06647d
	public static void main(String[] args)
	{
		HTML_Stegano html = new HTML_Stegano();
		int sp = 0;
		String s = "";

		try
        {
			System.out.println(">> HTML Stegano Coder");
			if(args.length == 2 && args[0].equals("-s"))
				System.out.println(html.calcSpace(args[1]) + " bit space");
			else if(args.length == 2 && args[0].equals("-d"))
				System.out.println(byteArrayToHexString(html.decodeMessage(args[1])));
			else if(args.length == 4 && args[0].equals("-c"))
				html.codeMessage(args[1], args[2], hexStringToByteArray(args[3]));
			else
				System.out.println("java HTML_Stegano -s infile\n\tjava HTML_Stegano -d infile\n\tjava HTML_Stegano -c infile outfile msg");

        } catch (Exception e)
        {
        	System.out.println("Ajjaj, gond a fjl rsa sorn!");
        	e.printStackTrace();
        }
	}

	public static byte[] hexStringToByteArray(String s)
	{
		if(s.length() % 2 != 0)
			return null;
		
		byte[] b = new byte[s.length()/2];
		int i2 = 0;
		
		for(int i = 0; i < s.length(); i += 2)
		{
			i2 = i/2;
			b[i2] = (byte) (hex.indexOf(s.charAt(i))*16);
			b[i2] = (byte) (b[i2] + (byte) hex.indexOf(s.charAt(i+1)));
		}
		
		return b;
	}
	
	public static String byteArrayToHexString(byte[] b)
	{
		StringBuffer s = new StringBuffer(b.length * 2);
		byte c;
		
		for(int i = 0; i < b.length; i++)
		{
			c = (byte) (b[i] & 0xF0);
	        c = (byte) (c >>> 4);
	        c = (byte) (c & 0x0F);
	        s.append(hex.charAt((int) c));
	        c = (byte) (b[i] & 0x0F); 
	        s.append(hex.charAt((int) c));
		}
		
		return new String(s);
	}
	
	public static byte binStringToByte(String s)
	{
		byte b = 0;
		
		if(s.length() != 8)
			return 0;
		
		while(s.length() > 0)
		{
			b <<= 1;
			if(s.charAt(0) == '1')
				b = (byte)(b | 0x01);

			s = s.substring(1);
		}
		
		return b;
	}
	
	public static String byteToBinString(byte b)
	{
		String s = "";
		
		while(s.length() < 8)
		{
			if(((byte)(b & 0x01)) == 0x01)
				s = "1" + s;
			else
				s = "0" + s;
			
			b >>= 1;
		}

		return s;
	}
}
